﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Media;

namespace FreeFall
{
    class FreeFall
    {
        private int gameInfoHeight;
        private bool gameover = false;
        private int score = 0;
        private int lives = 3;
        private int difficultyLevel = 0;
        private bool isCollision = false;
        SoundPlayer backgroundMusicPlayer;
        SoundPlayer gameSoundsPlayer;
        public void Init()
        {
            Console.CursorVisible = false;
            Console.ForegroundColor = ConsoleColor.White;
            Console.BufferHeight = Console.WindowHeight = 22;
            Console.BufferWidth = Console.WindowWidth = 30;
            gameInfoHeight = Console.WindowHeight - 5;
            gameSoundsPlayer = new SoundPlayer();
            backgroundMusicPlayer = new SoundPlayer();
        }

        private void playGameSound(string sound)
        {
            gameSoundsPlayer.SoundLocation = sound + ".wav";
            gameSoundsPlayer.Play();
            Thread.Sleep(300);
        }
        private void playBackgroundMusic()
        {
            backgroundMusicPlayer.SoundLocation = "sound.wav";
            backgroundMusicPlayer.PlayLooping();
        }

        public void Start()
        {
            DrawWelcomeScreen();
            StartGame();
        }

        private Player ChoosePlayer()
        {
            return new Player(Console.WindowWidth / 2 - 1, 1, "<*>", "/*\\");
        }

        private void StartGame()
        {
            while (true)
            {
                if (Console.KeyAvailable)
                {
                    if (Console.ReadKey(true).Key == ConsoleKey.Enter)
                    {
                        playGameSound("menu");
                        DrawDifficultyLevelSelection();
                        SetDifficultyAndPlay();
                    }
                }
            }
        }

        private void SetDifficultyAndPlay()
        {
            bool waitEnd = false;
            ConsoleKey key;
            while (!waitEnd)
            {
                if (Console.KeyAvailable)
                {
                    key = Console.ReadKey(true).Key;
                    if (key == ConsoleKey.D1 || key == ConsoleKey.NumPad1)
                    {
                        playGameSound("menu");
                        difficultyLevel = 0;
                        waitEnd = true;
                    }
                    else if (key == ConsoleKey.D2 || key == ConsoleKey.NumPad2)
                    {
                        playGameSound("menu");
                        difficultyLevel = 1;
                        waitEnd = true;
                    }
                    else if (key == ConsoleKey.D3 || key == ConsoleKey.NumPad3)
                    {
                        playGameSound("menu");
                        difficultyLevel = 2;
                        waitEnd = true;
                    }
                }
            }

            DrawMap();
        }

        private void DrawDifficultyLevelSelection()
        {
            Console.Clear();

            Console.WriteLine("\n\n");
            Console.WriteLine("       SELECT DIFFICULTY\n\n");
            Console.WriteLine("         Press number\n");
            Console.Write("\t---------------\n");
            Console.Write("          1. JUNIOR\n");
            Console.Write("\t---------------\n");
            Console.Write("\t---------------\n");
            Console.Write("          2.   MID\n");
            Console.Write("\t---------------\n");
            Console.Write("\t---------------\n");
            Console.Write("          3. SENIOR\n");
            Console.Write("\t---------------\n");
        }

        private void DrawMap()
        {
            Player player = ChoosePlayer();
            LinkedList<Obstacle> obstacles = new LinkedList<Obstacle>();
            Random randomGenerator = new Random();
            playBackgroundMusic();
            while (!gameover)
            {
                Console.Clear();
                DrawGameInfo();
                InitializeAndDrawObstacles(obstacles, player, randomGenerator);
                DrawPlayer(player);
                Thread.Sleep(150);
            }
            stopBackgroundMusic();
            DrawStatisics();
            playGameSound("gameOver");
            Restart();
        }

        private void stopBackgroundMusic()
        {
            backgroundMusicPlayer.Stop();
        }

        private void Restart()
        {
            gameover = false;
            score = 0;
            lives = 3;
            StartGame();
        }

        private void DrawStatisics()
        {
            Console.Clear();

            Console.WriteLine("\n\n\n");
            Console.WriteLine("\t   GAME OVER\n\n");
            Console.WriteLine("\t     Score\n");
            Console.Write("\t---------------\n");
            for (int i = 0; i < Console.WindowWidth / 2 - score.ToString().Length / 2; i++)
            {
                Console.Write(" ");
            }
            Console.WriteLine(score);
            Console.Write("\t---------------\n");

            Console.WriteLine("\n");
            Console.WriteLine("\t     Press");
            Console.Write("\t    -------\n");
            Console.Write("\t   | ENTER |\n");
            Console.Write("\t    -------\n");
            Console.WriteLine("\t to play again");
        }

        private void InitializeAndDrawObstacles(LinkedList<Obstacle> obstacles, Player player, Random randomGenerator)
        {
            GenerateObstacles(obstacles, randomGenerator);
            MoveObstaclesAndRemoveOutOfBoundsObstacles(obstacles, player);
            DrawObstacles(obstacles);
        }

        private void DrawGameInfo()
        {
            Console.SetCursorPosition(0, gameInfoHeight);
            Console.WriteLine("##############################");
            DrawScore();
            DrawLives();
        }

        private void DrawScore()
        {
            Console.SetCursorPosition(1, gameInfoHeight + 2);
            Console.WriteLine("Score: " + score);
        }

        private void DrawLives()
        {
            Console.SetCursorPosition(1, gameInfoHeight + 3);
            if(lives == 3)
            {
                Console.WriteLine("Lives: <3 <3 <3");
            }
            else if (lives == 2)
            {
                Console.WriteLine("Lives: <3 <3");
            }
            else if (lives == 1)
            {
                Console.WriteLine("Lives: <3");
            }
        }

        private void DrawWelcomeScreen()
        {
            Console.WriteLine("\n\n\n\n");
            Console.WriteLine("\t   FREE FALL\n\n");
            Console.WriteLine("\t     Press");
            Console.Write("\t    -------\n");
            Console.Write("\t   | ENTER |\n");
            Console.Write("\t    -------\n");
            Console.WriteLine("       to start the game");


            Console.WriteLine("\n\n");
            Console.WriteLine("\t  Use arrows");
            Console.WriteLine("       to controll game");
        }

        private void DrawPlayer(Player player)
        {
            MovePlayer(player);
            Console.SetCursorPosition(player.X, player.Y);
            if(isCollision)
            {
                Console.WriteLine(player.ShotAvatar);
                isCollision = false;
            } else
            {
                Console.WriteLine(player.Avatar);
            }
        }

        private void MovePlayer(Player player)
        {
            if (Console.KeyAvailable)
            {
                ConsoleKey key = Console.ReadKey().Key;
                if(key == ConsoleKey.RightArrow && (player.X + player.Avatar.Length < Console.WindowWidth))
                {
                    player.X++;
                } 
                else if(key == ConsoleKey.LeftArrow && (player.X > 0))
                {
                    player.X--;
                }
            }
        }

        private void GenerateObstacles(LinkedList<Obstacle> obstacles, Random random)
        {
            for (int i = 0; i < Console.WindowWidth; i++)
            {
                if (random.Next(0, 101) >= 100 - difficultyLevel)
                {
                    Obstacle newObstacle = new Obstacle(i, gameInfoHeight, ';');
                    obstacles.AddLast(newObstacle);
                }
            }
        }

        private void MoveObstaclesAndRemoveOutOfBoundsObstacles(LinkedList<Obstacle> obstacles, Player player)
        {
            List<Obstacle> obstacleToRemove = new List<Obstacle>();
            foreach (Obstacle obstacle in obstacles)
            {
                obstacle.Y -= 1;
                ObstacleOutOfBoundsCheck(obstacleToRemove, obstacle);
                CollisionCheck(player, obstacle, obstacleToRemove);
            }
            RemoveObstaclesOutOfBounds(obstacles, obstacleToRemove);
        }

        private void RemoveObstaclesOutOfBounds(LinkedList<Obstacle> obstacles, List<Obstacle> obstacleToRemove)
        {
            foreach (Obstacle obstacle in obstacleToRemove)
            {
                obstacles.Remove(obstacle);
            }
        }

        private void ObstacleOutOfBoundsCheck(List<Obstacle> obstacleToRemove, Obstacle obstacle)
        {
            if (obstacle.Y < 0)
            {
                obstacleToRemove.Add(obstacle);
                score++;
            }
        }

        private void CollisionCheck(Player player, Obstacle obstacle, List<Obstacle> obstacleToRemove)
        {
            if (Collision(obstacle, player))
            {
                isCollision = true;
                obstacleToRemove.Add(obstacle);
                lives--;
                if (lives <= 0)
                {
                    gameover = true;
                }
            }
        }

        private bool Collision(Obstacle obstacle, Player player)
        {
            if(obstacle.Y == player.Y)
            {
                if (obstacle.X >= player.X && obstacle.X < player.Avatar.Length + player.X)
                {
                    return true;
                }
            }
            return false;
        }

        private void DrawObstacles(LinkedList<Obstacle> rocks)
        {
            foreach (Obstacle obstacle in rocks)
            {
                Console.SetCursorPosition(obstacle.X, obstacle.Y);
                Console.Write(obstacle.Avatar);
            }
        }

    }
}
